@@ -47,6 +47,18 @@ class ITextSource(metaclass=ABCMeta):
     RFC822-like document came from.
     """
 
+    @abstractmethod
+    def relative_to(self, path):
+        """
+        Compute a new text source of the same kind with any filesystem
+        references translated to be relative from the specified base directory.
+
+        :param base_dir:
+            A base directory name
+        :returns:
+            A new text source
+        """
+
 
 class IJobDefinition(metaclass=ABCMeta):
     """
@@ -337,6 +349,21 @@ class IJobQualifier(metaclass=ABCMeta):
         :returns False: otherwise
         """
 
+    @abstractproperty
+    def origin(self):
+        """
+        Origin of this qualifier
+
+        :raises NonPrimitiveQualifierOrigin:
+            If the :meth:`is_primitive` property is True and this qualifier
+            has no well-defined origin of itself.
+
+        This property can be used to trace the origin of a qualifier back to
+        its definition point. Note that it may not always be available when
+        it doesn't make sense to say that a composite object came from any one
+        particular place.
+        """
+
 
 class IJobRunner(metaclass=ABCMeta):
     """
