@@ -28,6 +28,7 @@ Test definitions for plainbox.impl.run module
 import os
 import shutil
 import tempfile
+import warnings
 
 from collections import OrderedDict
 from inspect import cleandoc
@@ -46,6 +47,8 @@ class TestRun(TestCase):
 
     @patch.dict('sys.modules', {'concurrent': Mock()})
     def setUp(self):
+        warnings.filterwarnings(
+            'ignore', 'validate is deprecated since version 0.11')
         # session data are kept in XDG_CACHE_HOME/plainbox/.session
         # To avoid resuming a real session, we have to select a temporary
         # location instead
@@ -66,17 +69,19 @@ class TestRun(TestCase):
             self.assertEqual(call.exception.args, (0,))
         self.maxDiff = None
         expected = """
-        usage: plainbox run [-h] [--not-interactive] [-n] [-f FORMAT] [-p OPTIONS]
-                            [-o FILE] [-t TRANSPORT] [--transport-where WHERE]
-                            [--transport-options OPTIONS] [-i PATTERN] [-x PATTERN]
-                            [-w WHITELIST]
+        usage: plainbox run [-h] [--non-interactive] [-n] [--dont-suppress-output]
+                            [-f FORMAT] [-p OPTIONS] [-o FILE] [-t TRANSPORT]
+                            [--transport-where WHERE] [--transport-options OPTIONS]
+                            [-i PATTERN] [-x PATTERN] [-w WHITELIST]
 
         optional arguments:
           -h, --help            show this help message and exit
 
         user interface options:
-          --not-interactive     skip tests that require interactivity
+          --non-interactive     skip tests that require interactivity
           -n, --dry-run         don't really run most jobs
+          --dont-suppress-output
+                                don't suppress the output of certain job plugin types
 
         output options:
           -f FORMAT, --output-format FORMAT
@@ -157,3 +162,4 @@ class TestRun(TestCase):
     def tearDown(self):
         shutil.rmtree(self._sandbox)
         os.environ = self._env
+        warnings.resetwarnings()
