@@ -32,7 +32,6 @@ import io
 from pkg_resources import resource_string
 
 from plainbox.abc import IJobResult
-from plainbox.impl.exporter.xml import CONTROL_CODE_RE_BYTES
 from plainbox.impl.exporter.xml import CONTROL_CODE_RE_STR
 from plainbox.impl.exporter.xml import XMLSessionStateExporter, XMLValidator
 from plainbox.testing_utils import resource_json
@@ -60,25 +59,6 @@ class ControlCodeTests(TestCase):
         self.assertNotRegex('\t', CONTROL_CODE_RE_STR)
         self.assertNotRegex('\v', CONTROL_CODE_RE_STR)
 
-    def test_lower_range__bytes(self):
-        self.assertRegex(b'\x00', CONTROL_CODE_RE_BYTES)
-        self.assertRegex(b'\x1F', CONTROL_CODE_RE_BYTES)
-        # The lower range spans from 0..0x20 (space), exclusive
-        self.assertNotRegex(b'\x20', CONTROL_CODE_RE_BYTES)
-
-    def test_higher_range__bytes(self):
-        self.assertNotRegex(b'\x7E', CONTROL_CODE_RE_BYTES)
-        self.assertRegex(b'\x7F', CONTROL_CODE_RE_BYTES)
-        self.assertRegex(b'\x1F', CONTROL_CODE_RE_BYTES)
-        self.assertNotRegex(b'\xA0', CONTROL_CODE_RE_BYTES)
-
-    def test_explicitly_allowed__bytes(self):
-        self.assertNotRegex(b' ', CONTROL_CODE_RE_BYTES)
-        self.assertNotRegex(b'\n', CONTROL_CODE_RE_BYTES)
-        self.assertNotRegex(b'\r', CONTROL_CODE_RE_BYTES)
-        self.assertNotRegex(b'\t', CONTROL_CODE_RE_BYTES)
-        self.assertNotRegex(b'\v', CONTROL_CODE_RE_BYTES)
-
 
 class XMLSessionStateExporterTests(TestCaseWithParameters):
 
@@ -199,4 +179,4 @@ class XMLExporterTests(TestCase):
         expected_result = resource_string(
             "plainbox", "test-data/xml-exporter/example-data.xml"
         )  # unintuitively, resource_string returns bytes
-        self.assertEqual(self.actual_result, expected_result)
\ No newline at end of file
+        self.assertEqual(self.actual_result, expected_result)
