@@ -55,7 +55,9 @@ from plainbox.i18n import gettext as _
 from plainbox.impl.result import DiskJobResult
 from plainbox.impl.result import IOLogRecord
 from plainbox.impl.result import MemoryJobResult
+from plainbox.impl.secure.origin import Origin
 from plainbox.impl.secure.qualifiers import SimpleQualifier
+from plainbox.impl.session.state import SessionMetaData
 from plainbox.impl.session.state import SessionState
 
 logger = logging.getLogger("plainbox.session.resume")
@@ -92,7 +94,78 @@ class IncompatibleJobError(SessionResumeError):
     """
 
 
-class SessionResumeHelper:
+class EnvelopeUnpackMixIn:
+    """
+    A mix-in class capable of unpacking the envelope of the session storage
+    """
+
+    def unpack_envelope(self, data):
+        """
+        Unpack the binary envelope and get access to a JSON object
+
+        :param data:
+            Bytes representing the dormant session
+        :returns:
+            the JSON representation of a session stored in the envelope
+        :raises CorruptedSessionError:
+            if the representation of the session is corrupted in any way
+        """
+        try:
+            data = gzip.decompress(data)
+        except IOError:
+            raise CorruptedSessionError(_("Cannot decompress session data"))
+        try:
+            text = data.decode("UTF-8")
+        except UnicodeDecodeError:
+            raise CorruptedSessionError(_("Cannot decode session text"))
+        try:
+            return json.loads(text)
+        except ValueError:
+            raise CorruptedSessionError(_("Cannot interpret session JSON"))
+
+
+class SessionPeekHelper(EnvelopeUnpackMixIn):
+
+    def peek(self, data):
+        """
+        Peek at the meta-data of a dormant session.
+
+        :param data:
+            Bytes representing the dormant session
+        :returns:
+            a SessionMetaData object
+        :raises CorruptedSessionError:
+            if the representation of the session is corrupted in any way
+        :raises IncompatibleSessionError:
+            if session serialization format is not supported
+        """
+        json_repr = self.unpack_envelope(data)
+        return self._peek_json(json_repr)
+
+    def _peek_json(self, json_repr):
+        """
+        Resume a SessionMetaData object from the JSON representation.
+
+        This method is called by :meth:`peek()` after the initial envelope
+        and parsing is done. The only error conditions that can happen
+        are related to semantic incompatibilities or corrupted internal state.
+        """
+        logger.debug(_("Peeking at json... (see below)"))
+        logger.debug(json.dumps(json_repr, indent=4))
+        _validate(json_repr, value_type=dict)
+        version = _validate(json_repr, key="version", choice=[1])
+        if version == 1:
+            return SessionPeekHelper1().peek_json(json_repr)
+        elif version == 2:
+            return SessionPeekHelper2().peek_json(json_repr)
+        elif version == 3:
+            return SessionPeekHelper3().peek_json(json_repr)
+        else:
+            raise IncompatibleSessionError(
+                _("Unsupported version {}").format(version))
+
+
+class SessionResumeHelper(EnvelopeUnpackMixIn):
     """
     Helper class for implementing session resume feature.
 
@@ -139,18 +212,7 @@ class SessionResumeHelper:
         :raises IncompatibleJobError:
             if serialized jobs are not the same as current jobs
         """
-        try:
-            data = gzip.decompress(data)
-        except IOError:
-            raise CorruptedSessionError(_("Cannot decompress session data"))
-        try:
-            text = data.decode("UTF-8")
-        except UnicodeDecodeError:
-            raise CorruptedSessionError(_("Cannot decode session text"))
-        try:
-            json_repr = json.loads(text)
-        except ValueError:
-            raise CorruptedSessionError(_("Cannot interpret session JSON"))
+        json_repr = self.unpack_envelope(data)
         return self._resume_json(json_repr, early_cb)
 
     def _resume_json(self, json_repr, early_cb=None):
@@ -186,7 +248,7 @@ class ResumeDiscardQualifier(SimpleQualifier):
     """
 
     def __init__(self, jobs_repr):
-        super().__init__()
+        super().__init__(Origin.get_caller_origin())
         # Set of ids of jobs to retain (computed as keys of the
         # dictionary taken from the session resume representation)
         self._retain_id_set = frozenset(jobs_repr)
@@ -195,7 +257,143 @@ class ResumeDiscardQualifier(SimpleQualifier):
         return job.id not in self._retain_id_set
 
 
-class SessionResumeHelper1:
+class MetaDataHelper1MixIn:
+
+    @classmethod
+    def _restore_SessionState_metadata(cls, metadata, session_repr):
+        """
+        Extract meta-data information from the representation of the session
+        and set it in the given session object
+        """
+        # Get the representation of the meta-data
+        metadata_repr = _validate(
+            session_repr, key='metadata', value_type=dict)
+        # Set each bit back to the session
+        metadata.title = _validate(
+            metadata_repr, key='title', value_type=str, value_none=True)
+        metadata.flags = set([
+            _validate(
+                flag, value_type=str,
+                value_type_msg=_("Each flag must be a string"))
+            for flag in _validate(
+                metadata_repr, key='flags', value_type=list)])
+        metadata.running_job_name = _validate(
+            metadata_repr, key='running_job_name', value_type=str,
+            value_none=True)
+
+
+class MetaDataHelper2MixIn(MetaDataHelper1MixIn):
+
+    @classmethod
+    def _restore_SessionState_metadata(cls, metadata, session_repr):
+        """
+        Extract meta-data information from the representation of the session
+        and set it in the given session object
+        """
+        super()._restore_SessionState_metadata(metadata, session_repr)
+        # Get the representation of the meta-data
+        metadata_repr = _validate(
+            session_repr, key='metadata', value_type=dict)
+        app_blob = _validate(
+            metadata_repr, key='app_blob', value_type=str,
+            value_none=True)
+        if app_blob is not None:
+            try:
+                app_blob = app_blob.encode("ASCII")
+            except UnicodeEncodeError:
+                # TRANSLATORS: please don't translate app_blob
+                raise CorruptedSessionError(_("app_blob is not ASCII"))
+            try:
+                app_blob = base64.standard_b64decode(app_blob)
+            except binascii.Error:
+                # TRANSLATORS: please don't translate app_blob
+                raise CorruptedSessionError(_("Cannot base64 decode app_blob"))
+        metadata.app_blob = app_blob
+
+
+class MetaDataHelper3MixIn(MetaDataHelper2MixIn):
+
+    @classmethod
+    def _restore_SessionState_metadata(cls, metadata, session_repr):
+        """
+        Extract meta-data information from the representation of the session
+        and set it in the given session object
+        """
+        super()._restore_SessionState_metadata(metadata, session_repr)
+        # Get the representation of the meta-data
+        metadata_repr = _validate(
+            session_repr, key='metadata', value_type=dict)
+        metadata.app_id = _validate(
+            metadata_repr, key='app_id', value_type=str,
+            value_none=True)
+
+
+class SessionPeekHelper1(MetaDataHelper1MixIn):
+    """
+    Helper class for implementing session peek feature
+
+    This class works with data constructed by
+    :class:`~plainbox.impl.session.suspend.SessionSuspendHelper1` which has
+    been pre-processed by :class:`SessionPeekHelper` (to strip the initial
+    envelope).
+
+    The only goal of this class is to reconstruct session state meta-data.
+    """
+
+    def peek_json(self, json_repr):
+        """
+        Resume a SessionState object from the JSON representation.
+
+        This method is called by :meth:`peek()` after the initial envelope and
+        parsing is done. The only error conditions that can happen are related
+        to semantic incompatibilities or corrupted internal state.
+        """
+        _validate(json_repr, key="version", choice=[1])
+        session_repr = _validate(json_repr, key='session', value_type=dict)
+        metadata = SessionMetaData()
+        self._restore_SessionState_metadata(metadata, session_repr)
+        return metadata
+
+    def _build_SessionState(self, session_repr, early_cb=None):
+        """
+        Reconstruct the session state object.
+
+        This method creates a fresh SessionState instance and restores
+        jobs, results, meta-data and desired job list using helper methods.
+        """
+        logger.debug(_("Starting to restore metadata..."))
+        metadata = SessionMetaData()
+        self._peek_SessionState_metadata(metadata, session_repr)
+        return metadata
+
+
+class SessionPeekHelper2(MetaDataHelper2MixIn, SessionPeekHelper1):
+    """
+    Helper class for implementing session peek feature
+
+    This class works with data constructed by
+    :class:`~plainbox.impl.session.suspend.SessionSuspendHelper1` which has
+    been pre-processed by :class:`SessionPeekHelper` (to strip the initial
+    envelope).
+
+    The only goal of this class is to reconstruct session state meta-data.
+    """
+
+
+class SessionPeekHelper3(MetaDataHelper3MixIn, SessionPeekHelper2):
+    """
+    Helper class for implementing session peek feature
+
+    This class works with data constructed by
+    :class:`~plainbox.impl.session.suspend.SessionSuspendHelper1` which has
+    been pre-processed by :class:`SessionPeekHelper` (to strip the initial
+    envelope).
+
+    The only goal of this class is to reconstruct session state meta-data.
+    """
+
+
+class SessionResumeHelper1(MetaDataHelper1MixIn):
     """
     Helper class for implementing session resume feature
 
@@ -257,7 +455,8 @@ class SessionResumeHelper1:
             _("Starting to restore jobs and results to %r..."), session)
         self._restore_SessionState_jobs_and_results(session, session_repr)
         logger.debug(_("Starting to restore metadata..."))
-        self._restore_SessionState_metadata(session, session_repr)
+        self._restore_SessionState_metadata(session.metadata, session_repr)
+        logger.debug(_("restored metadata %r"), session.metadata)
         logger.debug(_("Starting to restore desired job list..."))
         self._restore_SessionState_desired_job_list(session, session_repr)
         logger.debug(_("Starting to restore job list..."))
@@ -371,29 +570,6 @@ class SessionResumeHelper1:
                 _("calling update_job_result(%r, %r)"), job, result_list[-1])
             session.update_job_result(job, result_list[-1])
 
-    @classmethod
-    def _restore_SessionState_metadata(cls, session, session_repr):
-        """
-        Extract meta-data information from the representation of the session
-        and set it in the given session object
-        """
-        # Get the representation of the meta-data
-        metadata_repr = _validate(
-            session_repr, key='metadata', value_type=dict)
-        # Set each bit back to the session
-        session.metadata.title = _validate(
-            metadata_repr, key='title', value_type=str, value_none=True)
-        session.metadata.flags = set([
-            _validate(
-                flag, value_type=str,
-                value_type_msg=_("Each flag must be a string"))
-            for flag in _validate(
-                metadata_repr, key='flags', value_type=list)])
-        session.metadata.running_job_name = _validate(
-            metadata_repr, key='running_job_name', value_type=str,
-            value_none=True)
-        logger.debug(_("restored metadata %r"), session.metadata)
-
     @classmethod
     def _restore_SessionState_desired_job_list(cls, session, session_repr):
         """
@@ -512,6 +688,44 @@ class SessionResumeHelper1:
         return IOLogRecord(delay, stream_name, data)
 
 
+class SessionResumeHelper2(MetaDataHelper2MixIn, SessionResumeHelper1):
+    """
+    Helper class for implementing session resume feature
+
+    This class works with data constructed by
+    :class:`~plainbox.impl.session.suspend.SessionSuspendHelper2` which has
+    been pre-processed by :class:`SessionResumeHelper` (to strip the initial
+    envelope).
+
+    Due to the constraints of what can be represented in a suspended session,
+    this class cannot work in isolation. It must operate with a list of know
+    jobs.
+
+    Since (most of the) jobs are being provided externally (as they represent
+    the non-serialized parts of checkbox or other job providers) several
+    failure modes are possible. Those are documented in :meth:`resume()`
+    """
+
+
+class SessionResumeHelper3(MetaDataHelper3MixIn, SessionResumeHelper2):
+    """
+    Helper class for implementing session resume feature
+
+    This class works with data constructed by
+    :class:`~plainbox.impl.session.suspend.SessionSuspendHelper3` which has
+    been pre-processed by :class:`SessionResumeHelper` (to strip the initial
+    envelope).
+
+    Due to the constraints of what can be represented in a suspended session,
+    this class cannot work in isolation. It must operate with a list of know
+    jobs.
+
+    Since (most of the) jobs are being provided externally (as they represent
+    the non-serialized parts of checkbox or other job providers) several
+    failure modes are possible. Those are documented in :meth:`resume()`
+    """
+
+
 def _validate(obj, **flags):
     """
     Multi-purpose extraction and validation function.
@@ -556,120 +770,3 @@ def _validate(obj, **flags):
                     obj_name, value_choice))
             raise CorruptedSessionError(error_msg)
     return value
-
-
-class SessionResumeHelper2(SessionResumeHelper1):
-    """
-    Helper class for implementing session resume feature
-
-    This class works with data constructed by
-    :class:`~plainbox.impl.session.suspend.SessionSuspendHelper2` which has
-    been pre-processed by :class:`SessionResumeHelper` (to strip the initial
-    envelope).
-
-    Due to the constraints of what can be represented in a suspended session,
-    this class cannot work in isolation. It must operate with a list of know
-    jobs.
-
-    Since (most of the) jobs are being provided externally (as they represent
-    the non-serialized parts of checkbox or other job providers) several
-    failure modes are possible. Those are documented in :meth:`resume()`
-    """
-
-    @classmethod
-    def _restore_SessionState_metadata(cls, session, session_repr):
-        """
-        Extract meta-data information from the representation of the session
-        and set it in the given session object
-        """
-        # Get the representation of the meta-data
-        metadata_repr = _validate(
-            session_repr, key='metadata', value_type=dict)
-        # Set each bit back to the session
-        session.metadata.title = _validate(
-            metadata_repr, key='title', value_type=str, value_none=True)
-        session.metadata.flags = set([
-            _validate(
-                flag, value_type=str,
-                value_type_msg=_("Each flag must be a string"))
-            for flag in _validate(
-                metadata_repr, key='flags', value_type=list)])
-        session.metadata.running_job_name = _validate(
-            metadata_repr, key='running_job_name', value_type=str,
-            value_none=True)
-        app_blob = _validate(
-            metadata_repr, key='app_blob', value_type=str,
-            value_none=True)
-        if app_blob is not None:
-            try:
-                app_blob = app_blob.encode("ASCII")
-            except UnicodeEncodeError:
-                # TRANSLATORS: please don't translate app_blob
-                raise CorruptedSessionError(_("app_blob is not ASCII"))
-            try:
-                app_blob = base64.standard_b64decode(app_blob)
-            except binascii.Error:
-                # TRANSLATORS: please don't translate app_blob
-                raise CorruptedSessionError(_("Cannot base64 decode app_blob"))
-        session.metadata.app_blob = app_blob
-        logger.debug(_("restored metadata %r"), session.metadata)
-
-
-class SessionResumeHelper3(SessionResumeHelper2):
-    """
-    Helper class for implementing session resume feature
-
-    This class works with data constructed by
-    :class:`~plainbox.impl.session.suspend.SessionSuspendHelper3` which has
-    been pre-processed by :class:`SessionResumeHelper` (to strip the initial
-    envelope).
-
-    Due to the constraints of what can be represented in a suspended session,
-    this class cannot work in isolation. It must operate with a list of know
-    jobs.
-
-    Since (most of the) jobs are being provided externally (as they represent
-    the non-serialized parts of checkbox or other job providers) several
-    failure modes are possible. Those are documented in :meth:`resume()`
-    """
-
-    @classmethod
-    def _restore_SessionState_metadata(cls, session, session_repr):
-        """
-        Extract meta-data information from the representation of the session
-        and set it in the given session object
-        """
-        # Get the representation of the meta-data
-        metadata_repr = _validate(
-            session_repr, key='metadata', value_type=dict)
-        # Set each bit back to the session
-        session.metadata.title = _validate(
-            metadata_repr, key='title', value_type=str, value_none=True)
-        session.metadata.flags = set([
-            _validate(
-                flag, value_type=str,
-                value_type_msg=_("Each flag must be a string"))
-            for flag in _validate(
-                metadata_repr, key='flags', value_type=list)])
-        session.metadata.running_job_name = _validate(
-            metadata_repr, key='running_job_name', value_type=str,
-            value_none=True)
-        app_blob = _validate(
-            metadata_repr, key='app_blob', value_type=str,
-            value_none=True)
-        if app_blob is not None:
-            try:
-                app_blob = app_blob.encode("ASCII")
-            except UnicodeEncodeError:
-                # TRANSLATORS: please don't translate app_blob
-                raise CorruptedSessionError(_("app_blob is not ASCII"))
-            try:
-                app_blob = base64.standard_b64decode(app_blob)
-            except binascii.Error:
-                # TRANSLATORS: please don't translate app_blob
-                raise CorruptedSessionError(_("Cannot base64 decode app_blob"))
-        session.metadata.app_blob = app_blob
-        session.metadata.app_id = _validate(
-            metadata_repr, key='app_id', value_type=str,
-            value_none=True)
-        logger.debug(_("restored metadata %r"), session.metadata)
