@@ -24,6 +24,7 @@
 import logging
 
 from plainbox.i18n import gettext as _
+from plainbox.impl import deprecated
 from plainbox.impl.depmgr import DependencyDuplicateError
 from plainbox.impl.depmgr import DependencyError
 from plainbox.impl.depmgr import DependencySolver
@@ -165,6 +166,176 @@ class SessionMetaData:
         self._app_id = value
 
 
+class SessionDeviceContext:
+    """
+    Session context specific to a given device.
+
+    This class exposes access to a "world view" unique to a specific device.
+    The view is composed of the following attributes:
+
+    :attr _provider_list:
+        A list of providers known by this device. All of those providers
+        are compatible with the device.
+
+    :attr _unit_list:
+        A list of all the units known by this device. Initially it is idenitcal
+        to the union of all the units from ``_provider_list`` but it is in fact
+        mutable and can be grown (or shrunk in some cases) when jobs are
+        created at runtime.
+
+    :attr _device:
+        Always None, this is a future extension point
+
+    :attr _state:
+        A :class:`SessionState` object that holds all of the job results
+        and also exposes some legacy API for computing the run list and the
+        desired job list
+    """
+
+    def __init__(self):
+        self._provider_list = []
+        self._unit_list = []
+        self._device = None
+        self._state = SessionState(self._unit_list)
+        # Connect SessionState's signals to fire our signals. This
+        # way all manipulation done through the SessionState object
+        # can be observed through the SessionDeviceContext object
+        # (and vice versa, as all the manipulation is forwarded to
+        # the SessionState)
+        self._state.on_unit_added.connect(self.on_unit_added)
+        self._state.on_unit_removed.connect(self.on_unit_removed)
+
+    @property
+    def device(self):
+        """
+        The device associated with this context.
+
+        .. warning::
+            Currently this method will always return None. In the future it
+            will return an object that describes the device.
+        """
+        return self._device
+
+    @property
+    def state(self):
+        """
+        The session state object associated with this context.
+
+        .. note::
+            You can use both the session state and the session device context
+            to query and monitor the changes to all the participating units
+        """
+        return self._state
+
+    @property
+    def provider_list(self):
+        """
+        The list of providers currently available in this context.
+
+        .. note::
+            You must not modify the return value.
+
+            This is not enforced but please use the :meth:`add_provider()`
+            method if you want to add a provider. Currently you cannot
+            remove providers or reorder the list of providers.
+        """
+        return self._provider_list
+
+    @property
+    def unit_list(self):
+        """
+        The list of units currently available in this context.
+
+        .. note::
+            You must not modify the return value.SessionDeviceContext:
+
+            This is not enforced but please use the :meth:`add_unit()`
+            or :meth:`remove_unit()` if you want to manipulate the list.
+            Currently you cannot reorder the list of units.
+        """
+        return self._unit_list
+
+    def add_provider(self, provider):
+        """
+        Add a provider to the context
+
+        :param provider:
+            The :class:`Provider1` to add
+        :raises ValueError:
+            If the provider is already in the context
+
+        This method can be used to add a provider to the context. It also adds
+        all of the units of that provider automatically.
+
+        .. note::
+            This method fires the :meth:`on_provider_added()` signal but
+            it does so before any of the units from that provider are added.
+        """
+        if provider in self._provider_list:
+            raise ValueError(_("attempting to add the same provider twice"))
+        self._provider_list.append(provider)
+        self.on_provider_added(provider)
+        for unit in provider.get_units()[0]:
+            self.add_unit(unit)
+
+    def add_unit(self, unit):
+        """
+        Add a unit to the context
+
+        :param unit:
+            The :class:`Unit` to add.
+        :raises ValueError:
+            If the unit is already in the context
+
+        This method can be used to register both the initially-known units
+        as well as units generated at runtime.
+
+        This method fires the :meth:`on_unit_added()` signal
+        """
+        if unit in self._unit_list:
+            raise ValueError(_("attempting to add the same unit twice"))
+        self.state.add_unit(unit)
+        # NOTE: no need to fire the on_unit_added() signal becuse the state
+        # object and we've connected it to will fire our version.
+
+    def remove_unit(self, unit):
+        """
+        Add an unit from the context
+
+        :param unit:
+            The :class:`Unit` to remove.
+
+        This method fires the :meth:`on_unit_removed()` signal
+        """
+        if unit not in self._unit_list:
+            raise ValueError(
+                _("attempting to remove unit not in this context"))
+        self.state.remove_unit(unit)
+        # NOTE: no need to fire the on_unit_removed() signal becuse the state
+        # object and we've connected it to will fire our version.
+
+    @Signal.define
+    def on_provider_added(self, provider):
+        """
+        Signal sent whenever a provider is added to the context.
+        """
+        logger.info(_("New provider added: %r"), provider)
+
+    @Signal.define
+    def on_unit_added(self, unit):
+        """
+        Signal sent whenever a unit is added to the context.
+        """
+        logger.info(_("Unit added: %r"), unit)
+
+    @Signal.define
+    def on_unit_removed(self, unit):
+        """
+        Signal sent whenever a unit is removed from the context.
+        """
+        logger.info(_("Unit removed: %r"), unit)
+
+
 class SessionState:
     """
     Class representing all state needed during a single program session.
@@ -289,6 +460,20 @@ class SessionState:
         """
         logger.info(_("Job removed: %r"), job)
 
+    @Signal.define
+    def on_unit_added(self, unit):
+        """
+        Signal sent whenever a unit is added to the session.
+        """
+        logger.info(_("Unit added: %r"), unit)
+
+    @Signal.define
+    def on_unit_removed(self, unit):
+        """
+        Signal sent whenever a unit is removed from the session.
+        """
+        logger.info(_("Unit removed: %r"), unit)
+
     def __init__(self, unit_list):
         """
         Initialize a new SessionState with a given list of units.
@@ -404,6 +589,7 @@ class SessionState:
             # And that each removed job was actually removed
             for job in remove_list:
                 self.on_job_removed(job)
+                self.on_unit_removed(job)
 
     def update_desired_job_list(self, desired_job_list):
         """
@@ -512,6 +698,7 @@ class SessionState:
         job.controller.observe_result(self, job, result)
         self._recompute_job_readiness()
 
+    @deprecated('0.9', 'use the add_unit() method instead')
     def add_job(self, new_job, recompute=True):
         """
         Add a new job to the session
@@ -543,6 +730,56 @@ class SessionState:
 
             This method recomputes job readiness for all jobs
         """
+        return self.add_unit(new_job, recompute)
+
+    def add_unit(self, new_unit, recompute=True):
+        """
+        Add a new unit to the session
+
+        :param new_unit:
+            The unit being added
+        :param recompute:
+            If True, recompute readiness inhibitors for all jobs.
+            You should only set this to False if you're adding
+            a number of jobs and will otherwise ensure that
+            :meth:`_recompute_job_readiness()` gets called before
+            session state users can see the state again.
+        :returns:
+            The unit that was actually added or an existing, identical
+            unit if a perfect clash was silently ignored.
+
+        :raises DependencyDuplicateError:
+            if a duplicate, clashing job definition is detected
+
+        .. note::
+            The following applies only to newly added job units:
+
+            The new_unit  gets added to all the state tracking objects of the
+            session. The job unit is initially not selected to run (it is not
+            in the desired_job_list and has the undesired inhibitor).
+
+            The new_unit job may clash with an existing job with the same id.
+            Unless both jobs are identical this will cause
+            DependencyDuplicateError to be raised. Identical jobs are silently
+            discarded.
+
+        .. note::
+            This method recomputes job readiness for all jobs unless the
+            recompute=False argument is used. Recomputing takes a while so if
+            you want to add a lot of units consider setting that to False and
+            only recompute at the last call.
+        """
+        if new_unit.Meta.name == 'job':
+            return self._add_job_unit(new_unit, recompute)
+        else:
+            return self._add_other_unit(new_unit)
+
+    def _add_other_unit(self, new_unit):
+        self.unit_list.append(new_unit)
+        self.on_unit_added(new_unit)
+        return new_unit
+
+    def _add_job_unit(self, new_job, recompute):
         # See if we have a job with the same id already
         try:
             existing_job = self.job_state_map[new_job.id].job
@@ -552,6 +789,7 @@ class SessionState:
             self.job_list.append(new_job)
             self.unit_list.append(new_job)
             self.on_job_state_map_changed()
+            self.on_unit_added(new_job)
             self.on_job_added(new_job)
             return new_job
         else:
@@ -566,6 +804,39 @@ class SessionState:
             if recompute:
                 self._recompute_job_readiness()
 
+    def remove_unit(self, unit, *, recompute=True):
+        """
+        Remove an existing unit from the session
+
+        :param unit:
+            The unit to remove
+        :param recompute:
+            If True, recompute readiness inhibitors for all jobs.
+            You should only set this to False if you're adding
+            a number of jobs and will otherwise ensure that
+            :meth:`_recompute_job_readiness()` gets called before
+            session state users can see the state again.
+
+        .. note::
+            This method recomputes job readiness for all jobs unless the
+            recompute=False argument is used. Recomputing takes a while so if
+            you want to add a lot of units consider setting that to False and
+            only recompute at the last call.
+        """
+        self._unit_list.remove(unit)
+        self.on_unit_removed(unit)
+        if unit.Meta.name == 'job':
+            self._job_list.remove(unit)
+            del self._job_state_map[unit.id]
+            try:
+                del self._resource_map[unit.id]
+            except KeyError:
+                pass
+            if recompute:
+                self._recompute_job_readiness()
+            self.on_job_removed(unit)
+            self.on_job_state_map_changed()
+
     def set_resource_list(self, resource_id, resource_list):
         """
         Add or change a resource with the given id.
