@@ -27,6 +27,7 @@ import re
 
 from plainbox.abc import IJobDefinition
 from plainbox.i18n import gettext as _
+from plainbox.i18n import gettext_noop as N_
 from plainbox.impl.resource import ResourceProgram
 from plainbox.impl.resource import parse_imports_stmt
 from plainbox.impl.secure.origin import JobOutputTextSource
@@ -86,7 +87,8 @@ class propertywithsymbols(property):
         function.
         """
         return propertywithsymbols(
-            fget, self.fset, self.fdel, self.__doc__, symbols=self.symbols)
+            fget, self.fset, self.fdel, self.__doc__ or fget.__doc__,
+            symbols=self.symbols)
 
 
 class _PluginValues(SymbolDef):
@@ -197,14 +199,6 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
         """
         return 'job'
 
-    def tr_unit(self):
-        """
-        Translated (optionally) value of the unit field (overridden)
-
-        The return value is always 'job' (translated)
-        """
-        return _("job")
-
     @property
     def partial_id(self):
         """
@@ -471,9 +465,11 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
             raise ValueError(_("record.origin.source.job must be this job"))
         return self.from_rfc822_record(record, self.provider)
 
-    class Meta(UnitWithId.Meta, JobDefinitionLegacyAPI.Meta):
+    class Meta:
 
-        class fields(UnitWithId.Meta.fields):
+        name = N_('job')
+
+        class fields(SymbolDef):
             """
             Symbols for each field that a JobDefinition can have
             """
@@ -492,9 +488,7 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
             flags = 'flags'
             category_id = 'category_id'
 
-        field_validators = {}
-        field_validators.update(UnitWithId.Meta.field_validators)
-        field_validators.update({
+        field_validators = {
             fields.name: [
                 UntranslatableFieldValidator,
                 TemplateVariantFieldValidator,
@@ -699,7 +693,4 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
                     ),
                     onlyif=lambda unit: unit.command),
             ]
-        })
-
-
-JobDefinition.fields = JobDefinition.Meta.fields
+        }
