@@ -21,6 +21,7 @@
 =================================================
 """
 
+import abc
 import collections
 import hashlib
 import json
@@ -30,7 +31,10 @@ import string
 from plainbox.i18n import gettext as _
 from plainbox.impl.secure.origin import Origin
 from plainbox.impl.secure.rfc822 import normalize_rfc822_value
+from plainbox.impl.symbol import Symbol
 from plainbox.impl.symbol import SymbolDef
+from plainbox.impl.symbol import SymbolDefMeta
+from plainbox.impl.symbol import SymbolDefNs
 from plainbox.impl.unit import get_accessed_parameters
 from plainbox.impl.unit._legacy import UnitLegacyAPI
 from plainbox.impl.unit.validators import IFieldValidator
@@ -125,29 +129,34 @@ class UnitValidator:
                         "{}.Meta.fields[{!r}][{}] is not a validator"
                     ).format(unit.__class__.__name__, field, index))
 
-    def advice(self, unit, field, kind, message=None, offset=0):
+    def advice(self, unit, field, kind, message=None, *, offset=0,
+               origin=None):
         """
         Shortcut for :meth:`report_issue` with severity=Severity.advice
         """
         return self.report_issue(
-            unit, field, kind, Severity.advice, message, offset)
+            unit, field, kind, Severity.advice, message,
+            offset=offset, origin=origin)
 
-    def warning(self, unit, field, kind, message=None, offset=0):
+    def warning(self, unit, field, kind, message=None, *, offset=0,
+                origin=None):
         """
         Shortcut for :meth:`report_issue` with severity=Severity.warning
         """
         return self.report_issue(
-            unit, field, kind, Severity.warning, message, offset)
+            unit, field, kind, Severity.warning, message,
+            offset=offset, origin=origin)
 
-    def error(self, unit, field, kind, message=None, offset=0):
+    def error(self, unit, field, kind, message=None, *, offset=0, origin=None):
         """
         Shortcut for :meth:`report_issue` with severity=Severity.error
         """
         return self.report_issue(
-            unit, field, kind, Severity.error, message, offset)
+            unit, field, kind, Severity.error, message,
+            offset=offset, origin=origin)
 
     def report_issue(self, unit, field, kind, severity, message=None,
-                     offset=0):
+                     *, offset=0, origin=None):
         """
         Helper method that aids in adding issues
 
@@ -168,9 +177,13 @@ class UnitValidator:
             This argument is required if :meth:`explain()` doesn't know
             about the specific value of ``kind`` used
         :param offset:
-            An (optional) offset within the field itself. This optional
-            argument can be used to point to a specific line in a multi-line
+            An (optional, keyword-only) offset within the field itself.
+            If specified it is used to point to a specific line in a multi-line
             field.
+        :param origin:
+            An (optional, keyword-only) origin to use to report the issue.
+            If specified it totally overrides all implicit origin detection.
+            The ``offset`` is not applied in this case.
         :returns:
             The reported issue
         :raises ValueError:
@@ -186,26 +199,30 @@ class UnitValidator:
         # compute the origin
         if isinstance(unit, list):
             cls = MultiUnitFieldIssue
-            origin = unit[0].origin
-            if field in unit[0].field_offset_map:
-                origin = origin.with_offset(
-                    unit[0].field_offset_map[field] + offset
-                ).just_line()
-            elif '_{}'.format(field) in unit[0].field_offset_map:
-                origin = origin.with_offset(
-                    unit[0].field_offset_map['_{}'.format(field)] + offset
-                ).just_line()
+            if origin is None:
+                origin = unit[0].origin
+                if field in unit[0].field_offset_map:
+                    origin = origin.with_offset(
+                        unit[0].field_offset_map[field] + offset
+                    ).just_line()
+                elif '_{}'.format(field) in unit[0].field_offset_map:
+                    if origin is None:
+                        origin = origin.with_offset(
+                            unit[0].field_offset_map['_{}'.format(field)]
+                            + offset).just_line()
         else:
             cls = UnitFieldIssue
-            origin = unit.origin
-            if field in unit.field_offset_map:
-                origin = origin.with_offset(
-                    unit.field_offset_map[field] + offset
-                ).just_line()
-            elif '_{}'.format(field) in unit.field_offset_map:
-                origin = origin.with_offset(
-                    unit.field_offset_map['_{}'.format(field)] + offset
-                ).just_line()
+            if origin is None:
+                origin = unit.origin
+                if field in unit.field_offset_map:
+                    origin = origin.with_offset(
+                        unit.field_offset_map[field] + offset
+                    ).just_line()
+                elif '_{}'.format(field) in unit.field_offset_map:
+                    if origin is None:
+                        origin = origin.with_offset(
+                            unit.field_offset_map['_{}'.format(field)]
+                            + offset).just_line()
         issue = cls(message, severity, kind, origin, unit, field)
         self.issue_list.append(issue)
         return issue
@@ -240,7 +257,102 @@ class UnitValidator:
     }
 
 
-class Unit(UnitLegacyAPI):
+class UnitType(abc.ABCMeta):
+    """
+    Meta-class for all Units
+
+    This metaclass is responsible for collecting meta-data about particular
+    units and exposing them in the special 'Meta' attribute of each class.
+
+    It also handles Meta inheritance so that SubUnit.Meta inherits from
+    Unit.Meta even if it was not specified directly.
+    """
+
+    def __new__(mcls, name, bases, ns):
+        # mro = super().__new__(mcls, name, bases, ns).__mro__
+        base_meta_list = [
+            base.Meta for base in bases if hasattr(base, 'Meta')]
+        our_meta = ns.get('Meta')
+        if our_meta is not None and base_meta_list:
+            new_meta_ns = dict(our_meta.__dict__)
+            new_meta_ns['__doc__'] = """
+                Collection of meta-data about :class:`{}`
+
+                This class is partially automatically generated.
+                It always inherits the Meta class of the base unit type.
+
+                This class has (at most) three attributes:
+
+                    `field_validators`:
+                        A dictionary mapping from each field to a list of
+                        :class:`IFieldvalidator:` that check that particular
+                        field for correctness.
+
+                    `fields`:
+                        A :class`SymbolDef` with a symbol for each field that
+                        this unit defines. This does not include dynamically
+                        created fields that are not a part of the unit itself.
+
+                    `validator_cls`:
+                        A :class:`UnitValidator` subclass that can be used to
+                        check this unit for correctness
+            """.format(name)
+            new_meta_bases = tuple(base_meta_list)
+            # Merge custom field_validators with base unit validators
+            if 'field_validators' in our_meta.__dict__:
+                merged_validators = dict()
+                for base_meta in base_meta_list:
+                    if hasattr(base_meta, 'field_validators'):
+                        merged_validators.update(base_meta.field_validators)
+                merged_validators.update(our_meta.field_validators)
+                new_meta_ns['field_validators'] = merged_validators
+            # Merge fields with base unit fields
+            if 'fields' in our_meta.__dict__:
+                # Look at all the base Meta classes and collect each
+                # Meta.fields class as our (real) list of base classes.
+                assert our_meta.fields.__bases__ == (SymbolDef,)
+                merged_fields_bases = [
+                    base_meta.fields
+                    for base_meta in base_meta_list
+                    if hasattr(base_meta, 'fields')]
+                # If there are no base classes then let's just inherit from the
+                # base SymbolDef class (not that we're actually ignoring any
+                # base classes on the our_meta.fields class as it can only be
+                # SymbolDef and nothing else is supported or makes sense.
+                if not merged_fields_bases:
+                    merged_fields_bases.append(SymbolDef)
+                # The list of base fields needs to be a tuple
+                merged_fields_bases = tuple(merged_fields_bases)
+                # Copy all of the Symbol objects out of the our_meta.field
+                # class that we're re-defining.
+                merged_fields_ns = SymbolDefNs()
+                for sym_name in dir(our_meta.fields):
+                    sym = getattr(our_meta.fields, sym_name)
+                    if isinstance(sym, Symbol):
+                        merged_fields_ns[sym_name] = sym
+                merged_fields_ns['__doc__'] = """
+                A symbol definition containing all fields used by :class:`{}`
+
+                This class is partially automatically generated. It always
+                inherits from the Meta.fields class of the base unit class.
+                """.format(name)
+                # Create a new class in place of the 'fields' defined in
+                # our_meta.fields.
+                fields = SymbolDefMeta(
+                    'fields', merged_fields_bases, merged_fields_ns)
+                fields.__qualname__ = '{}.Meta.fields'.format(name)
+                new_meta_ns['fields'] = fields
+            # Ensure that Meta.name is explicitly defined
+            if 'name' not in our_meta.__dict__:
+                raise TypeError(_(
+                    "Please define 'name' in {}.Meta"
+                ).format(name))
+            ns['Meta'] = type('Meta', new_meta_bases, new_meta_ns)
+        ns['fields'] = ns['Meta'].fields
+        return super().__new__(mcls, name, bases, ns)
+
+
+class Unit(UnitLegacyAPI, metaclass=UnitType):
     """
     Units are representations of data loaded from RFC822 definitions
 
@@ -256,7 +368,7 @@ class Unit(UnitLegacyAPI):
     """
 
     def __init__(self, data, raw_data=None, origin=None, provider=None,
-                 parameters=None, field_offset_map=None):
+                 parameters=None, field_offset_map=None, virtual=False):
         """
         Initialize a new unit
 
@@ -281,6 +393,11 @@ class Unit(UnitLegacyAPI):
         :param field_offset_map:
             An optional dictionary with offsets (in line numbers) of each
             field.  Line numbers are relative to the value of origin.line_start
+        :param virtual:
+            An optional flag marking this unit as "virtual". It can be used
+            to annotate units synthetized by PlainBox itself so that certain
+            operations can treat them differently. It also helps with merging
+            non-virtual and virtual units.
         """
         if raw_data is None:
             raw_data = data
@@ -295,6 +412,7 @@ class Unit(UnitLegacyAPI):
         self._provider = provider
         self._checksum = None
         self._parameters = parameters
+        self._virtual = virtual
 
     @classmethod
     def instantiate_template(cls, data, raw_data, origin, provider, parameters,
@@ -339,12 +457,13 @@ class Unit(UnitLegacyAPI):
         """
         return self.get_record_value('unit')
 
-    @property
     def tr_unit(self):
         """
-        Translated (optionally) value of the unit field
+        Translated (optionally) value of the unit field (overridden)
+
+        The return value is always 'self.Meta.name' (translated)
         """
-        return self.get_record_value('unit', _("unit"))
+        return _(self.Meta.name)
 
     @property
     def origin(self):
@@ -384,6 +503,16 @@ class Unit(UnitLegacyAPI):
         """
         return self._parameters
 
+    @property
+    def virtual(self):
+        """
+        Flag indicating if this unit is a virtual unit
+
+        Virtual units are created (synthetised) by PlainBox and don't exist
+        in any one specific file as normal units do.
+        """
+        return self._virtual
+
     @property
     def is_parametric(self):
         """
@@ -676,10 +805,12 @@ class Unit(UnitLegacyAPI):
             for issue in validator.check_in_context(self, context):
                 yield issue
 
-    class Meta(UnitLegacyAPI.Meta):
+    class Meta:
         """
         Class containing additional meta-data about this unit.
 
+        :attr name:
+            Name of this unit as it can appear in unit definition files
         :attr fields:
             A :class:`plainbox.impl.symbol.SymbolDef` with a symbol for each of
             the fields used by this unit.
@@ -689,6 +820,8 @@ class Unit(UnitLegacyAPI):
             A dictionary mapping each field to a list of field validators
         """
 
+        name = 'unit'
+
         class fields(SymbolDef):
             """
             Unit defines only one field, the 'unit'
