@@ -38,6 +38,7 @@ from plainbox.impl.validation import Severity
 __all__ = [
     'CorrectFieldValueValidator',
     'DeprecatedFieldValidator',
+    'FieldValidatorBase',
     'IFieldValidator',
     'PresentFieldValidator',
     'TemplateInvariantFieldValidator',
@@ -392,7 +393,8 @@ class TranslatableFieldValidator(FieldValidatorBase):
     """
 
     def check(self, parent, unit, field):
-        if (unit.get_record_value(field)
+        if (unit.virtual is False
+                and unit.get_record_value(field) is not None
                 and not unit.is_translatable_field(field)):
             yield parent.warning(unit, field, Problem.expected_i18n)
 
@@ -489,11 +491,12 @@ class ShellProgramValidator(FieldValidatorBase):
                     if token is not None:
                         yield parent.error(
                             unit, field, Problem.syntax_error,
-                            "{}, near {!r}".format(exc, token), lex.lineno - 1)
+                            "{}, near {!r}".format(exc, token),
+                            offset=lex.lineno - 1)
                     else:
                         yield parent.error(
                             unit, field, Problem.syntax_error, str(exc),
-                            lex.lineno - 1)
+                            offset=lex.lineno - 1)
 
 
 def compute_value_map(context, field):
@@ -513,7 +516,10 @@ def compute_value_map(context, field):
         *(provider.get_units()[0]
           for provider in context.provider_list))
     for unit in all_units:
-        value = getattr(unit, field2prop(field))
+        try:
+            value = getattr(unit, field2prop(field))
+        except AttributeError:
+            continue
         if value not in value_map:
             value_map[value] = [unit]
         else:
