@@ -373,10 +373,13 @@ class ProviderManagerToolTests(TestCase):
             "\tdescription: description\n"
             "\tversion: 1.0\n"
             "\tgettext domain: domain\n"
-            "[Unit Definitions]\n"
+            "[Job Definitions]\n"
             "\tjob 2014.com.example::dummy, from jobs/jobs.txt:1-4\n"
-            "[White Lists]\n"
-            "\t'test', from whitelists/test.whitelist:1\n"
+            "[Test Plans]\n"
+            "\ttest plan 2014.com.example::test, from whitelists/test.whitelist:1\n"
+            "[Other Units]\n"
+            "\tfile jobs/jobs.txt, role unit-source\n"
+            "\tfile whitelists/test.whitelist, role legacy-whitelist\n"
             "[Executables]\n"
             "\t'test.sh'\n"))
 
